# PyOpenAl
# Copyright (C) 2003 Jean-Baptiste LAMY -- jiba@tuxfamily.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""pyopenal.openal4soya

A module for interfacing OpenAL with the Soya 3D engine
(http://oomadness.tuxfamily.org/en/soya).

See Soya tutorial lesson 116 for more info."""

import pyopenal, os, os.path
from pyopenal import *

PATH = ""

_BUFFERS  = {}
_LISTENER = None
_CAMERA   = None

def init(camera, freq = 44100):
  """init(camera, freq = 44100)

Inits OpenAL for Soya. CAMERA will be used to compute the position of the listener ;
it is not required that it is a Camera, though it is usually one."""
  
  pyopenal.init()
  
  global _LISTENER, _CAMERA
  if not _LISTENER:
    _LISTENER          = Listener(freq)
    _LISTENER.position = (0.0, 0.0,  0.0)
    _LISTENER.at       = (0.0, 0.0, -1.0)
    _LISTENER.up       = (0.0, 1.0,  0.0)
    
  _CAMERA = camera

_SOURCES_POS_SPEED = []

def play(filename, position = None, speed = None, looping = 0):
  """play(filename, position = None, speed = None, looping = 0)

Plays sound FILENAME, at the given POSITION and SPEED (for Doppler effect).
If LOOPING is true, the sound is played in loop.
Sound file's data are cached for more efficiency.

FILENAME can be Wave or Ogg Vorbis file (need PyOgg and PyVorbis)."""
  buffer = _BUFFERS.get(filename)
  if not buffer:
    if   filename.endswith(".wav"): buffer = _BUFFERS[filename] = WaveBuffer     (os.path.join(PATH, filename))
    elif filename.endswith(".ogg"): buffer = _BUFFERS[filename] = OggVorbisBuffer(os.path.join(PATH, filename))
    
  source          = Source()
  source.buffer   = buffer
  source.looping  = looping
  
  _SOURCES_POS_SPEED.append((source, position, speed))
  
  if position:
    source.position = _CAMERA.transform_point(position.x, position.y, position.z, position.parent)
    
    if speed:
      source.velocity = _CAMERA.transform_vector(speed.x, speed.y, speed.z, speed.parent)
      
  source.play()
  
  return source


def _begin_round():
  """_begin_round()

Update the sources's positions and speed.

This function is called before each Soya rendering."""
  i = 0
  while i < len(_SOURCES_POS_SPEED):
    source, position, speed = _SOURCES_POS_SPEED[i]
    
    if source.get_state() == AL_STOPPED:
      del _SOURCES_POS_SPEED[i]
      
    else:
      i += 1
      if position:
        source.position = _CAMERA.transform_point(position.x, position.y, position.z, position.parent)
        
        if speed:
          source.velocity = _CAMERA.transform_vector(speed.x, speed.y, speed.z, speed.parent)
      

import soya

_old_render = soya.render
def render():
  _begin_round()
  _old_render()

soya.render = render
